package Kephra::API::Panel;
use strict;
use warnings;

our $VERSION = '0.00';

=head1 NAME

Kephra::API::Panel - API for App parts with own Panel

=head1 DESCRIPTION

Not yet specced, but this are parts of the app that are visual and
functional units of its own, with an own panel and own submenu in the 
modules-menu like e.g. a file manager.

=cut



1;
